//notation: js file can only use this kind of comments
//since comments will cause error when use in webview.loadurl,
//comments will be remove by java use regexp
(function() {

    if (window.WebViewJavascriptBridge) {
         return;
    }

    var messagingIframe;
    var sendMessageQueue = [];
    var receiveMessageQueue = [];
    var messageHandlers = {};

    var CUSTOM_PROTOCOL_SCHEME = 'yy';
    var QUEUE_HAS_MESSAGE = '__QUEUE_MESSAGE__/';

    var responseCallbacks = {};
    var uniqueId = 1;

    function _createQueueReadyIframe(doc) {
        messagingIframe = doc.createElement('iframe');
        messagingIframe.style.display = 'none';
        doc.documentElement.appendChild(messagingIframe);
    }

    //set default messageHandler
    function init(messageHandler) {
        if (WebViewJavascriptBridge._messageHandler) {
            throw new Error('WebViewJavascriptBridge.init called twice');
        }
        WebViewJavascriptBridge._messageHandler = messageHandler;
        var receivedMessages = receiveMessageQueue;
        receiveMessageQueue = null;
        for (var i = 0; i < receivedMessages.length; i++) {
            _dispatchMessageFromNative(receivedMessages[i]);
        }
    }

    function send(data, responseCallback) {
        _doSend({
            data: data
        }, responseCallback);
    }

    function registerHandler(handlerName, handler) {
        messageHandlers[handlerName] = handler;
    }

    function callHandler(handlerName, data, responseCallback) {
        _doSend({
            handlerName: handlerName,
            data: data
        }, responseCallback);
    }

    //sendMessage add message, 触发native处理 sendMessage
    function _doSend(message, responseCallback) {
        // console.log("_doSend(), handlerName: " + message.handlerName);
        //console.log("_doSend(), data: " + message.data);
        if (responseCallback) {
            var callbackId = 'cb_' + (uniqueId++) + '_' + new Date().getTime();
            // console.log("_doSend(), callbackId: " + callbackId);
            responseCallbacks[callbackId] = responseCallback;
            message.callbackId = callbackId;
        }
        sendMessageQueue.push(message);
        // console.log("_doSend(), messagingIframe.src: " + CUSTOM_PROTOCOL_SCHEME + '://' + QUEUE_HAS_MESSAGE);
        messagingIframe.src = CUSTOM_PROTOCOL_SCHEME + '://' + QUEUE_HAS_MESSAGE;
    }

    // 提供给native调用,该函数作用:获取sendMessageQueue返回给native,由于android不能直接获取返回的内容,所以使用url shouldOverrideUrlLoading 的方式返回内容
    function _fetchQueue() {
        var messageQueueString = JSON.stringify(sendMessageQueue);
        sendMessageQueue = [];
        //android can't read directly the return data, so we can reload iframe src to communicate with java
        messagingIframe.src = CUSTOM_PROTOCOL_SCHEME + '://return/_fetchQueue/' + encodeURIComponent(messageQueueString);
    }

    // 提供给native使用,
    function _dispatchMessageFromNative(messageJSON) {
        var message = JSON.parse(messageJSON);
        var responseCallback;
        // console.log("_dispatchMessageFromNative, 000000, message.responseId: " + message.responseId);
        //java call finished, now need to call js callback function
        if (message.responseId) {
            // console.log("_dispatchMessageFromNative, 11111, message.responseId: " + message.responseId);
            responseCallback = responseCallbacks[message.responseId];
            if (!responseCallback) {
                return;
            }
            responseCallback(message.responseData);
            delete responseCallbacks[message.responseId];
        } else {
            // 直接发送
            if (message.callbackId) {
                // console.log("_dispatchMessageFromNative, 22222, message.callbackId: " + message.callbackId);
                var callbackResponseId = message.callbackId;
                responseCallback = function(responseData) {
                    _doSend({
                        responseId: callbackResponseId,
                        responseData: responseData
                    });
                };
            }

            var handler = WebViewJavascriptBridge._messageHandler;
            if (message.handlerName) {
                handler = messageHandlers[message.handlerName];
            }
            // 查找指定handler
            try {
                handler(message.data, responseCallback);
            } catch (exception) {
                if (typeof console != 'undefined') {
                    console.log("WebViewJavascriptBridge: WARNING: javascript handler threw.", message, exception);
                }
            }
        }
    }

    // 提供给native调用,receiveMessageQueue 在会在页面加载完后赋值为null,所以
    function _handleMessageFromNative(messageJSON) {
        // console.log("_handleMessageFromNative: " + messageJSON);
        if (receiveMessageQueue && receiveMessageQueue.length > 0) {
            // console.log("receiveMessageQueue.push()");
            receiveMessageQueue.push(messageJSON);
        } else {
            // console.log("_dispatchMessageFromNative(), messageJSON: " + messageJSON);
            _dispatchMessageFromNative(messageJSON);
        }
    }

    var WebViewJavascriptBridge = window.WebViewJavascriptBridge = {
        init: init,
        send: send,
        closeWebWindow: closeWebWindow,
        openWebWindow: openWebWindow,
        showSharePannel: showSharePannel,
        showQRSharePannel: showQRSharePannel,
        fingerPrintLockStatus: fingerPrintLockStatus,

        deviceInfo: deviceInfo,
        keyStoreIO: keyStoreIO,

        saveValueAndKey: saveValueAndKey,
        saveCacheValueAndKey: saveCacheValueAndKey,
        cacheValueForKey: cacheValueForKey,
        valueForKey: valueForKey,
        removeCacheValueForKey: removeCacheValueForKey,
        removeValueForKey: removeValueForKey,

        getFinStatus: getFinStatus,

        gestureLockStatus: gestureLockStatus,
        closeGestureLock: closeGestureLock,
        createGestureLock: createGestureLock,

        registerHandler: registerHandler,
        callNativeRefresh: callNativeRefresh,
        gotoPage: gotoPage,
        setStatusBarColor: setStatusBarColor,
        loginSuccess: loginSuccess,
        logoutSuccess: logoutSuccess,
        showNavigation: showNavigation,
        showFundDetailLine: showFundDetailLine,
        gotoMarket: gotoMarket,
        showPasswordView: showPasswordView,
        closePasswordView: closePasswordView,
        showPDFView: showPDFView,
        copyString: copyString,
        showQualifiedInvestorPage: showQualifiedInvestorPage,
        resetRiskMatchStatus: resetRiskMatchStatus,
        showLockPage: showLockPage,
        appOnEvent: appOnEvent,
        appOnEventWithDuration: appOnEventWithDuration,
        appEventStart: appEventStart,
        appEventEnd: appEventEnd,
        requestByApp: requestByApp,
        loginByApp: loginByApp,
        showMapView: showMapView,
        updateUserInfo: updateUserInfo,
        loginMobileByApp: loginMobileByApp,
        openNativePage: openNativePage,
        callHandler: callHandler,
        _fetchQueue: _fetchQueue,
        _handleMessageFromNative: _handleMessageFromNative
    };

    var doc = document;
    _createQueueReadyIframe(doc);
    var readyEvent = doc.createEvent('Events');
    readyEvent.initEvent('WebViewJavascriptBridgeReady');
    readyEvent.bridge = WebViewJavascriptBridge;
    doc.dispatchEvent(readyEvent);

    function openWebWindow(data, responseCallback) {
        callHandler('openWebWindow', data,responseCallback);
    }

    function closeWebWindow(data, responseCallback) {
        callHandler('closeWebWindow', data, responseCallback);
    }

    function showSharePannel(data, responseCallback) {
        callHandler('showSharePannel', data, responseCallback);
    }

    function showQRSharePannel(data, responseCallback) {
            callHandler('showQRSharePannel', data, responseCallback);
        }

    function fingerPrintLockStatus(data, responseCallback) {
        callHandler('fingerPrintLockStatus', data, responseCallback);
    }

    function deviceInfo(data, responseCallback) {
        callHandler('deviceInfo', data, responseCallback);
    }

    function keyStoreIO(data, responseCallback) {
        callHandler('keyStoreIO', data, responseCallback);
    }

    function shareInfo(data, responseCallback) {
        callHandler('shareInfo', data,responseCallback);
    }

    function makePhoneCall(data, responseCallback) {
        callHandler('makePhoneCall', data, responseCallback);
    }

    function netWorkStatue(data, responseCallback) {
        callHandler('netWorkStatue', data, responseCallback);
    }

    function createGestureLock(data, responseCallback) {
        callHandler('createGestureLock', data, responseCallback);
    }

    function closeGestureLock(data, responseCallback) {
        callHandler('closeGestureLock', data, responseCallback);
    }

    function gestureLockStatus(data, responseCallback) {
        callHandler('gestureLockStatus', data, responseCallback);
    }

    function getFinStatus(param,myF){
        callHandler('getFinStatus', data, responseCallback);
    }

    function saveCacheValueAndKey(data, responseCallback) {
        callHandler('saveCacheValueAndKey',data,responseCallback);
    }

    function saveValueAndKey(data, responseCallback) {
        callHandler('saveValueAndKey', data,responseCallback);
    }

    function cacheValueForKey(data, responseCallback) {
        callHandler('cacheValueForKey', data,responseCallback);
    }

    function valueForKey(data, responseCallback) {
        callHandler('valueForKey', data,responseCallback);
    }

    function removeCacheValueForKey(data, responseCallback) {
        callHandler('removeCacheValueForKey', data,responseCallback);
    }

    function removeValueForKey(data, responseCallback) {
        callHandler('removeValueForKey', data,responseCallback);
    }

    function openCamera(data, responseCallback) {
        callHandler('openCamera', data,responseCallback);
    }

    function showGestureLock(data, responseCallback) {
        callHandler('showGestureLock', data,responseCallback);
    }

    function callNativeRefresh(data, responseCallback) {
        callHandler('callNativeRefresh', data,responseCallback);
    }

    function gotoPage(data, responseCallback) {
        callHandler('gotoPage', data,responseCallback);
    }

    function setStatusBarColor(data, responseCallback) {
        callHandler('setStatusBarColor', data,responseCallback);
    }

    function loginSuccess(data, responseCallback) {
        callHandler('loginSuccess', data,responseCallback);
    }

    function logoutSuccess(data, responseCallback) {
        callHandler('logoutSuccess', data,responseCallback);
    }

    function showNavigation(data, responseCallback) {
        callHandler('showNavigation', data,responseCallback);
    }

    function showFundDetailLine(data, responseCallback) {
        callHandler('showFundDetailLine', data,responseCallback);
    }

    function gotoMarket(data, responseCallback) {
        callHandler('gotoMarket', data,responseCallback);
    }

    function showPasswordView(data, responseCallback) {
        callHandler('showPasswordView', data,responseCallback);
    }

    function closePasswordView(data, responseCallback) {
        callHandler('closePasswordView', data,responseCallback);
    }

    function showPDFView(data, responseCallback) {
        callHandler('showPDFView', data,responseCallback);
    }

    function copyString(data, responseCallback) {
        callHandler('copyString', data,responseCallback);
    }

    function showQualifiedInvestorPage(data, responseCallback) {
        callHandler('showQualifiedInvestorPage', data,responseCallback);
    }

    function resetRiskMatchStatus(data, responseCallback) {
        callHandler('resetRiskMatchStatus', data,responseCallback);
    }

    function showLockPage(data, responseCallback) {
        callHandler('showLockPage', data,responseCallback);
    }

    function appOnEvent(data, responseCallback) {
        callHandler('appOnEvent', data,responseCallback);
    }

    function appOnEventWithDuration(data, responseCallback) {
        callHandler('appOnEventWithDuration', data,responseCallback);
    }

    function appEventStart(data, responseCallback) {
        callHandler('appEventStart', data,responseCallback);
    }

    function appEventEnd(data, responseCallback) {
        callHandler('appEventEnd', data,responseCallback);
    }

    function requestByApp(data, responseCallback) {
        callHandler('requestByApp', data,responseCallback);
    }

    function loginByApp(data, responseCallback) {
        callHandler('loginByApp', data,responseCallback);
    }

    function showMapView(data, responseCallback) {
        callHandler('showMapView', data,responseCallback);
    }

    function updateUserInfo(data, responseCallback) {
        callHandler('updateUserInfo', data,responseCallback);
    }
    function loginMobileByApp(data, responseCallback)
    {
        callHandler('loginMobileByApp',data.responseCallback);
    }
    function openNativePage(data, responseCallback)
    {
        callHandler('openNativePage',data,responseCallback);
    }
})();
